/*
 *  Author: Enrico Forestieri <forenr@tlc.unipr.it>
 *  Created:  Sat Sep  4 18:29:56  2004
 *  Modified: Sun Apr 16 15:33:18  2006
 *     - Account for spaces in gsprint path.
 *
 *  Modify and redistribute freely. Use at your own risk.
 *
 *  Program to print postscript files to a windows printer through
 *  gsprint ( http://www.cs.wisc.edu/~ghost/gsview/gsprint.htm ).
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <sys/stat.h>

#define BUFSIZE 256
#define CMDSIZE 400

/* Look for gsview in the registry to find the path to gsprint */
char *
find_gsprint(char *cbuf, char *pbuf, char *tmpfile, char *tmpdir)
{
    char *start, *end;
    FILE *fp;
    struct stat st;
    static char gsprint[BUFSIZE];

    sprintf(tmpfile, "%s\\gsprint.uni", tmpdir);
    sprintf(cbuf, "regedit /e \"%s\" HKEY_LOCAL_MACHINE\\SOFTWARE\\Ghostgum\\Gsview", tmpfile);
    system(cbuf);
    if (stat(tmpfile,&st) == 0)
    {   /* tmpfile may be in unicode, hence use type to convert it */
	sprintf(cbuf, "type \"%s\" > \"%s\\gsprint.txt\"", tmpfile, tmpdir);
	system(cbuf);
	remove(tmpfile);
	sprintf(tmpfile, "%s\\gsprint.txt", tmpdir);

	if ((fp=fopen(tmpfile,"r")))
	{
	   gsprint[0] = '\0';
	   while (fgets(pbuf,BUFSIZE,fp))
	   {
	       if ((start=strstr(pbuf, "\"=\"")))
	       {
		   start += 3;
		   end = strrchr(pbuf, '"');
		   if (start < end)
		   {
		       int i = 0;
		       while (start < end)
		       {
			   if (*start == '\\' && *(start+1) == '\\')
			       start++;
			   cbuf[i++] = *start++;
		       }
		       cbuf[i] = '\0';
		       strcat(cbuf, "\\gsview\\gsprint.exe");
		       if (stat(cbuf,&st) == 0)
			   strcpy(gsprint,cbuf);
		   }
	       }
	   }
	   fclose(fp);
	   remove(tmpfile);
	   if (gsprint[0])
	       return gsprint;
	}
    }
    return NULL;
}

int
main(int ac, char **av)
{
    int c;
    extern int optind;
    int errflg=0;
    FILE *fp;
    char *tmpdir;
    char *gsprint;
    char tmpfile[BUFSIZE];
    char pbuf[BUFSIZE];
    char cbuf[CMDSIZE];

    while ((c = getopt(ac, av, "P:d:h")) != EOF)
    {
	switch (c)
	{
	    case 'P':
	    case 'd':
	    case 'h':
		break;
	    default:
		errflg++;
	}
    }

    if (errflg)
    {
	fprintf(stderr,"Prints the given files to a windows printer by using gsprint.\n\n");
	fprintf(stderr,"Usage: lp [-h] [-d|-P printer] <files>...\n");
	fprintf(stderr,"All switches are accepted for compatibility with lp or lpr but are ignored.\n");
	fprintf(stderr,"If gsprint is found, its printer dialog is shown, otherwise an error is issued.\n");
	exit(1);
    }


    if (!(tmpdir=getenv("TEMP")))
    {
        fprintf(stderr, "%s: the environment variable %%TEMP%% is not set\n", av[0]);
        exit(1);
    }

    if (!(gsprint=find_gsprint(cbuf,pbuf,tmpfile,tmpdir)))
    {
        fprintf(stderr, "%s: cannot find gsprint.exe\n", av[0]);
        exit(1);
    }

    strcpy(cbuf,"\"\"");
    strcat(cbuf,gsprint);
    strcat(cbuf,"\" -q -query ");
    
    if (optind==ac)
    {	/* Read from stdin and create a temp file */
        sprintf(tmpfile, "%s\\gsprint.ps", tmpdir);
	if ((fp=fopen(tmpfile,"w")))
	{
	    while (fgets(pbuf,sizeof(pbuf),stdin))
		fputs(pbuf,fp);
	    fclose(fp);
	}
	else
	{
            fprintf(stderr, "%s: cannot open temporary file\n", av[0]);
	    exit(1);
	}
	strcat(cbuf,"\"");
	strcat(cbuf,tmpfile);
	strcat(cbuf,"\"\"");
	system(cbuf);
	remove(tmpfile);
    }
    else
    {	/* Remaining args on cmd line are treated as files to print */
	int l = strlen(cbuf);
	while (optind<ac)
	{
	    cbuf[l] = 0;
	    strcat(cbuf,"\"");
	    strcat(cbuf,av[optind]);
	    strcat(cbuf,"\"\"");
	    fprintf(stderr, "%s\n", cbuf);
	    system(cbuf);
	    optind++;
	}
    }
    return 0;
}
